# by ddzeko & ico, Fri, 18 Mar 2005 14:44:08 +0100
cp_get_ifaddr() {
  [ "$CP_SCRIPT_DEBUG" ] && set -vx
  local ifaddr interface echo_return
  echo_return="$CP_ECHO_RETURN" # since we're making our own calls
  # in our own way we need to set CP_ECHO_RETURN the way we want it
  # but preserving initial state so it could be used afterwards
  CP_ECHO_RETURN=""

  interface="$1"
  if [ -z "$interface" ]; then
    cp_get_ifdefault dev
    interface="$RET"
    [ -z "$interface" ] && interface=lo
  fi

  if ! ifconfig $interface 2> /dev/null >> /dev/null; then
    echo "cp_get_ifaddr: $interface: No such interface" 1>&2
    CP_ECHO_RETURN="$echo_return"
    return 1
  fi

  ifaddr="`/sbin/ifconfig $interface | awk '/inet/{ printf("%s\n",substr($2,index($2,":")+1)) }'`"

  if [ -z $ifaddr ]; then
    echo "cp_get_ifaddr: $interface: No such ipaddress" 1>&2
    CP_ECHO_RETURN="$echo_return"
    return 1
  fi

  RET="$ifaddr"
  [ -z "$echo_return" ] || echo $RET
  CP_ECHO_RETURN="$echo_return"
}

# by ddzeko & ico, Fri, 18 Mar 2005 14:44:08 +0100
cp_get_ifmask() {
  [ "$CP_SCRIPT_DEBUG" ] && set -vx
  local ifmask interface echo_return
  echo_return="$CP_ECHO_RETURN" # since we're making our own calls
  # in our own way we need to set CP_ECHO_RETURN the way we want it
  # but preserving initial state so it could be used afterwards
  CP_ECHO_RETURN=""

  interface="$1"
  if [ -z "$interface" ]; then
    cp_get_ifdefault dev
    interface="$RET"
    [ -z "$interface" ] && interface=lo
  fi

  if ! ifconfig $interface 2> /dev/null >> /dev/null; then
    echo "cp_get_ifmask: $interface: No such interface" 1>&2
    CP_ECHO_RETURN="$echo_return"
    return 1
  fi

  ifmask="`/sbin/ifconfig $interface | awk '/Mask/{if($3~/Mask/)a=$3;else if ($4~/Mask/)a=$4;printf ("%s\n", substr(a,index(a,":")+1))}'`"

  if [ -z "$ifmask" ]; then
    echo "cp_get_ifmask: $interface: No such netmask" 1>&2
    CP_ECHO_RETURN="$echo_return"
    return 1
  fi

  RET="$ifmask"
  [ -z "$echo_return" ] || echo $RET
  CP_ECHO_RETURN="$echo_return"
}

# by ddzeko, Mon, 21 Mar 2005 07:00:22 +0100
cp_get_ifdefault() {
  local echo_return
  echo_return="$CP_ECHO_RETURN" # since we're making our own calls
  # in our own way we need to set CP_ECHO_RETURN the way we want it
  # but preserving initial state so it could be used afterwards
  CP_ECHO_RETURN=""

  [ "$CP_SCRIPT_DEBUG" ] && set -vx
  RET=""
  case $1 in
    dev)  RET=`route -n | awk '/^0.0.0.0/{print $8}'` ;;
    addr) RET=`route -n | awk '/^0.0.0.0/{print $2}'` ;;
    *)    echo "cp_get_ifdefault: Argument required (dev or addr)" >&2 ;;
  esac
  [ -z "$RET" -a "$1" -a "$CP_VERBOSE" ] && echo "cp_get_ifdefault: No default route" >&2
  [ -z "$echo_return" ] || echo $RET
  CP_ECHO_RETURN="$echo_return"
  [ -z "$RET" ] && return 1
}

# by ico, Tue, 15 Mar 2005 14:04:21 +0100
cp_get_netaddr() {
  [ "$CP_SCRIPT_DEBUG" ] && set -vx
  local netmask ipaddress interface cidr echo_return
  echo_return="$CP_ECHO_RETURN" # since we're making our own calls
  # in our own way we need to set CP_ECHO_RETURN the way we want it
  # but preserving initial state so it could be used afterwards
  CP_ECHO_RETURN=""

  interface="$1"
  if [ -z "$interface" ]; then
    cp_get_ifdefault dev
    interface="$RET"
    [ -z "$interface" ] && interface=lo
  fi

  if ! ifconfig $interface 2> /dev/null >> /dev/null; then
    echo "cp_get_netaddr: $interface: No such interface" 1>&2
    CP_ECHO_RETURN="$echo_return"
    return 1
  fi

  cp_get_ifaddr "$interface"
  ipaddress="$RET"
  if [ -z $ipaddress ]; then
    echo "cp_get_netaddr: $interface: No such ipaddress" 1>&2
    CP_ECHO_RETURN="$echo_return"
    return 1
  fi
  cp_get_ifmask "$interface"
  netmask="$RET"
  if [ -z $netmask ]; then
    echo "cp_get_netaddr: $interface: No such netmask" 1>&2
    CP_ECHO_RETURN="$echo_return"
    return 1
  fi

  if [ "$netmask" = "255.255.255.255" ]; then
    cidr="$ipaddress/32"
  else 
    cidr="`ipcalc -ncb $ipaddress $netmask | awk '/^Network:/{print $2}'`"
  fi
  
  RET="$cidr"
  [ -z "$echo_return" ] || echo $RET
  CP_ECHO_RETURN="$echo_return"
}

# by ico, Tue, 15 Mar 2005 14:04:21 +0100
cp_backup_conffile() {
  [ "$CP_SCRIPT_DEBUG" ] && set -vx
  local file_bak

  if [ -z "$1" ]; then
    return 1
  fi
  if [ ! -f "$1" ]; then
    echo "cp_backup_conffile: $1: No such file" 1>&2
    return 1
  fi

  file_bak="/var/backups/`basename $1`.bak"

  if [ ! -f $file_bak ]; then
    cp -pf "$1" $file_bak
  else
    if ! cmp -s "$1" $file_bak; then
      /usr/bin/savelog -p -c 7 $file_bak > /dev/null 2> /dev/null
      cp -pf $1 $file_bak
    fi
  fi
}

# by jelly, Tue, 15 Mar 2005 14:04:21 +0100
# A sed wrapper, to use instead of perl -pi -e
#  - relatively safe in-place s///g
#  - takes care of symlinks and ownership
# returns true if changed, false if nothing happened
#
cp_check_and_sed() {
  [ "$CP_SCRIPT_DEBUG" ] && set -vx
  local s sedcmd ret i
  s="$1"
  shift
  sedcmd="$1"
  shift
  ret=2
  for i in "$@"
  do
    [ -e "$i" ]        || continue
    egrep -q "$s" "$i" || continue
    [ -h "$i" ]        && i=$(readlink -f "$i")
    sed "$sedcmd" "$i" > "$i.dpkg-tmp"
    if ! cmp -s "$i" "$i.dpkg-tmp" 2>&1 >/dev/null; then
      cp_mv "$i.dpkg-tmp" "$i"
    else
      rm "$i.dpkg-tmp"
    fi
    ret=0
  done
  return $ret
}

# by jelly, Sun, 20 Mar 2005 20:12:19 +0100
cp_echo () {
  [ "$CP_SCRIPT_DEBUG" ] && set -vx
  if [ "x$1" = "x-mailonly" -o "x$1" = "x-m" ]; then
    shift
  else   
    echo "$*"
  fi
  CP_NOTICE="$CP_NOTICE$1"
  if [ `echo -n "$*" | wc -l` -eq 0 ]; then
    CP_NOTICE="$CP_NOTICE
"
  fi
}   

# by jelly, Sun, 20 Mar 2005 20:12:19 +0100
cp_mail () {
  [ "$CP_SCRIPT_DEBUG" ] && set -vx
  local pkg version quiet
  if [ "x$1" = "x-q" ]; then
    quiet=1
    shift
  fi
  [ -n "$1" ] && pkg="$1"       # barf if no package
  [ -n "$2" ] && version=" $2" || true
  if [ "$(echo $CP_NOTICE|wc -w)" -gt 0 ]; then
    [ -n "$quiet" ] || echo "Mailing upgrade output to root."
    CP_NOTICE="From: $pkg postinst script <root>
To: root
Subject: $pkg$version package install log
Date: $(/bin/date +'%a, %d %b %Y %H:%M:%S %z')
 
Hello!

The $pkg package has been successfully installed on your computer.
For your convenience, a partial output of the last $pkg installation
is included below.

----------
${CP_NOTICE}----------"
  
    echo "$CP_NOTICE" | /usr/sbin/sendmail -t &
    sleep 1
  fi
}

# by ddzeko, Mon, 21 Mar 2005 11:31:59 +0100
cp_mv () {
  [ "$CP_SCRIPT_DEBUG" ] && set -vx
  local new old
  if [ -z "$2" ]; then
    return 1
  fi
  new="$1"
  old="$2"
  if [ -e "$old" ]; then
    chown --reference "$old" "$new"
    chmod --reference "$old" "$new"
  fi
  mv "$new" "$old"
}
# by ddzeko, Mon, 21 Mar 2005 13:35:42 +0100
cp_yes_no () {
  [ "$CP_SCRIPT_DEBUG" ] && set -vx
  local prompt answer
  prompt="$1"
  [ "$prompt" ] || prompt="Are you sure?"
  RET=""

  # check to prevent user from using this when debconf is active
  # (it would break communication with its backend - uses stdin/out)
  if [ "$DEBCONF_REDIR" ]; then
    echo "cp_yes_no: debconf redirection detected" >&2
    return 1
  fi

  echo -n "$prompt (y)es/(n)o: "
  read answer
  case "$answer" in
    y*)
      echo
      RET=y
      ;;
    n*)
      echo
      RET=n
      ;;
    *)
      echo
      echo "Please read the message and choose y or n"
      cp_yes_no
      ;;
  esac
}

# by ddzeko, Tue, 29 Mar 2005 17:37:52 +0200
#
# db_get seems to return error messages if the requested variable 
# is not found - we do not find that desirable, and checking $? with
# set -e turned on seems to be a bit of a problem
# --> that's why we wrap the db_get call to return unset RET variable
#     instead of some gibberish error message that our script does
#     not expect
cp_db_get() {
  local var
  var="$1"
  
  # check to prevent user from using this before activating debconf
  # confmodule needs to be loaded before accessing db_* functions
  if [ -z "$DEBCONF_REDIR" ]; then
    echo "cp_db_get: debconf not activated!" >&2
    return 1
  fi
 
  if ! db_get "$var"; then
    case "$RET" in
        *doesn\'t\ exist)
          RET=""
          ;;
        [0-9][0-9]\ ?*) # other errors
          RET=""
          ;;
    esac
  fi
}

